vim9script

# Vim :new command and class constructors.
class Test
	def new()
	enddef
	def newOther()
	enddef
	def newyetanother()
	enddef
endclass

Test.new()
Test.newOther()
Test.newyetanother()
new
quit
